"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

meta = {
	'name': 'Bing Search',
	'author': 'Saeed',
	'version': '0.2',
	'description': 'Search your query in the bing.com and show the results.',
	'sources': ('bing',),
	'options': (
		('query', None, True, 'Query string', '-q', 'store', str),
		('limit', 1, False, 'Search limit(number of pages, default=1)', '-l', 'store', int),
		('count', 50, False, 'Number of results per page(min=10, max=100, default=50)', '-c', 'store', int),
	),
	'examples': ('bing -q <QUERY> -l 15 --output --api',)
}

def module_api(self):
	query = self.options['query']
	limit = self.options['limit']
	count = self.options['count']
	run = self.bing(query, limit, count)
	run.run_crawl()
	results = run.results
	output = {'results': results}
	self.save_gather(output, 'search/bing', query, output=self.options['output'])
	return output

def module_run(self):
	self.search_engine_results(module_api(self))



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

class main:

	def __init__(self, q, limit=15):
		""" piratebay search engine
			q         : query for search
			limit     : maximum result count
		"""
		self.framework = main.framework
		self.q = self.framework.urlib(q).quote_plus
		self.max = limit
		self._json = ''
		self._results = []

	def run_crawl(self):
		url = 'https://apibay.org/q.php'
		payload = {'q' : self.q, 'cat':''}
		self.framework.verbose('[PIRATEBAY] Searching piratebay...')
		try:
			req = self.framework.request(
					url=url,
					params=payload,
					allow_redirects=True)
		except:
			self.framework.error('ConnectionError.', 'util/piratebay', 'run_crawl')
			self.framework.error('Piratebay is missed!', 'util/piratebay', 'run_crawl')

		else:
			self._rawhtml = req.text
			self._json = req.json()

	def make_magnet(self, info_hash, name):
		return (f"magnet:?xt=urn:btih:{info_hash}&dn={name}"
			'&tr=udp://tracker.coppersurfer.tk:6969/announce'
			'&tr=udp://open.demonii.com:1337'
			'&tr=udp://tracker.openbittorrent.com:6969/announce'
			'&tr=udp://tracker.opentrackr.org:1337'
			'&tr=udp://tracker.leechers-paradise.org:6969/announce'
			'&tr=udp://tracker.dler.org:6969/announce'
			'&tr=udp://opentracker.i2p.rocks:6969/announce'
			'&tr=udp://47.ip-51-68-199.eu:6969/announce')

	@property
	def raw(self):
		return self._rawhtml

	@property
	def json(self):
		return self._json

	def format_size(self, num, suffix='B'):
		for unit in ['','Ki','Mi','Gi','Ti','Pi','Ei','Zi']:
			if abs(num) < 1024.0:
				return f"{num:3.1f} {unit}{suffix}"
			num /= 1024.0
		return f"{num:.1f} Yi{suffix}"

	@property
	def results(self):
		for count,torrent in enumerate(self.json):
			if count == self.max:
				break

			title = torrent['name']
			info = torrent['info_hash']
			seedleech = f"seeders: {torrent['seeders']} | leechers: {torrent['leechers']}"
			size = self.format_size(int(torrent['size']))

			self._results.append({
				't': title,
				'a': self.make_magnet(info, title),
				'c': f"{torrent['username']} | {size}",
				'd': seedleech
				})

		return self._results



"""
OWASP Maryam!
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
# core/util/iris/topic.py
# Hatma Suryotrisongko

import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
from sentence_transformers import SentenceTransformer

class main:

	def __init__(self, inputfile, filetype, keyword, showcharts, verbose):

		from dask import dataframe as dd
		import json
		from gensim.parsing.preprocessing import remove_stopwords

		if verbose == True:
			print("\n\n DATASET = reading file : " + inputfile)
			print("\n\n Search keyword = " + keyword)

		if filetype == "csv":
			# tmp = pd.read_csv(inputfile, header=None, low_memory=False)
			tmp = dd.read_csv(inputfile, sep=';', header=None)
			tmp2 = tmp.to_dask_array(lengths=True)
			tmp3 = tmp2.compute()
			tmp4 = pd.DataFrame(tmp3)

			if verbose == True:
				print("\n\n csv file (before preprocessing) = ")
				print(tmp4)

			self.corpus = tmp4[0].str.lower().apply(remove_stopwords).to_numpy()

		elif filetype == "json":
			with open(inputfile) as json_file:
				jsonfile = json.load(json_file)

			tmp = pd.DataFrame(jsonfile['results'])

			if verbose == True:
				print("\n\n json file (before preprocessing) = ")
				print(tmp)

			tmp['td'] = tmp['t'] + ' ' + tmp['d']
			self.corpus = tmp['td'].str.lower().apply(remove_stopwords).to_numpy()

		else:
			print('ERROR, only accept csv or json file!')

		if verbose == True:
			print('\n\n number of corpus = ')
			print(len(self.corpus))
			print('\n\n self.corpus[0] = ')
			print(self.corpus[0])
			print('\n\n all self.corpus = ')
			print(self.corpus)

		if showcharts == True:
			print('\n\n histogram of the number of words in each corpus')
			pd.Series([len(e.split()) for e in self.corpus]).hist()
			plt.show()

	def run_sklearn_cluster_kmeans(self, selected_pretrained_model, showcharts, verbose):

		from sklearn.cluster import KMeans
		import scipy
		import umap

		pretrained_model = selected_pretrained_model
		if verbose == True:
			print('\n\n Model selection')
			# https://www.sbert.net/docs/pretrained_models.html
			print(pretrained_model)

		model = SentenceTransformer(pretrained_model)
		if verbose == True:
			print(model)

		corpus_embeddings = model.encode(self.corpus)
		if verbose == True:
			print('\n\n CORPUS EMBEDDING')
			print(corpus_embeddings.shape)
			print(corpus_embeddings)

		K = 5
		kmeans = KMeans(n_clusters=5, random_state=0).fit(corpus_embeddings)
		if verbose == True:
			print('\n\n Show Cluster using SkLearn KMeans')
			print(kmeans)

		corpus_labeled = pd.DataFrame({'ClusterLabel': kmeans.labels_, 'Sentence': self.corpus})
		print('\n\n corpus_labeled = ')
		print(corpus_labeled)

		cls_dist = pd.Series(kmeans.labels_).value_counts()
		if verbose == True:
			print('\n\n frequency of cluster label = ')
			print(cls_dist)

		distances = scipy.spatial.distance.cdist(kmeans.cluster_centers_, corpus_embeddings)
		if verbose == True:
			print("\n\n calculate distance of cluster's center point = ")
			print(distances)

		print("\n\n Cluster's center example = ")

		centers = {}
		print('Cluster', 'Size', 'Center-idx', 'Center-Example', sep='\t\t')
		for i, d in enumerate(distances):
			ind = np.argsort(d, axis=0)[0]
			centers[i] = ind
			print(i, cls_dist[i], ind, self.corpus[ind], sep="\t\t")

		if showcharts == True:
			print('\n\n Visualization of the cluster points')

			X = umap.UMAP(n_components=2, min_dist=0.0).fit_transform(corpus_embeddings)
			labels = kmeans.labels_

			fig, ax = plt.subplots(figsize=(12, 8))
			plt.scatter(X[:, 0], X[:, 1], c=labels, s=1, cmap='Paired')
			for c in centers:
				plt.text(X[centers[c], 0], X[centers[c], 1], 'CLS-' + str(c), fontsize=24)
			plt.colorbar()
			plt.show()

	def run_topic_modeling_bertopic(self, selected_pretrained_model, verbose):

		from bertopic import BERTopic

		pretrained_model = selected_pretrained_model
		if verbose == True:
			print('\n\n Model selection')
			# https://www.sbert.net/docs/pretrained_models.html
			print(pretrained_model)

		model = SentenceTransformer(pretrained_model)
		if verbose == True:
			print(model)

		corpus_embeddings = model.encode(self.corpus)
		if verbose == True:
			print('\n\n CORPUS EMBEDDING')
			print(corpus_embeddings.shape)
			print(corpus_embeddings)

		print('\n\n Topic Modeling with BERTopic')

		sentence_model = SentenceTransformer(pretrained_model)
		if verbose == True:
			print(sentence_model)

		topic_model = BERTopic(embedding_model=sentence_model)
		if verbose == True:
			print(topic_model)

		topics, _ = topic_model.fit_transform(self.corpus)
		print(topic_model.get_topic_info()[:6])
		output = topic_model.get_topic_info()

		corpus_labeled = pd.DataFrame({'ClusterLabel': topics, 'Sentence': self.corpus})
		if verbose == True:
			print("\n\n corpus_labeled = ")
			print(corpus_labeled)

		print('\n\n topics for each cluster = ')

		i = 0
		while i < len(topic_model.get_topic_info()):
			print(f"Cluster #{i} = ")
			print(topic_model.get_topic(i))
			i += 1

		return output


	def run_search_topics_top2vec(self, keyword, showcharts, verbose):

		from top2vec import Top2Vec

		print('\n\n Search Topics Using Top2Vec (caution: might not work well for a small dataset)')
		print('\n the Search Keyword = ' + keyword)

		pretrained_embedding_model = 'universal-sentence-encoder-multilingual'
		if verbose == True:
			print('\n\n Pretrained Embedding Model')
			# https://tfhub.dev/google/universal-sentence-encoder-multilingual/
			# 16 languages (Arabic, Chinese-simplified, Chinese-traditional, English, French, German, Italian, Japanese, Korean, Dutch, Polish, Portuguese, Spanish, Thai, Turkish, Russian) text encoder.
			print(pretrained_embedding_model)

		model = Top2Vec(documents=self.corpus.tolist(), speed='learn', workers=8)
		if verbose == True:
			print('\n Model = ')
			print(model)

		if model.get_num_topics() < 5:
			ntopics = model.get_num_topics()
		else:
			ntopics = 5

		topic_words, word_scores, topic_nums = model.get_topics(ntopics)
		print(topic_words)
		print(word_scores)
		print(topic_nums)

		print('\n Semantic Search Documents by Keywords = ')
		documents, document_scores, document_ids = model.search_documents_by_keywords(keywords=[keyword], num_docs=5)
		for doc, score, doc_id in zip(documents, document_scores, document_ids):
			print(f"Document: {doc_id}, Score: {score}")
			print('-----------')
			print(doc)
			print('-----------')
			print()

		if showcharts == True:
			print('\n\n Generate Word Clouds = ')
			topic_words, word_scores, topic_scores, topic_nums = model.search_topics(keywords=[keyword], num_topics=ntopics)
			for topic in topic_nums:
				model.generate_topic_wordcloud(topic)

		print('\n Similar Keywords = ')
		words, word_scores = model.similar_words(keywords=[keyword], keywords_neg=[], num_words=20)
		for word, score in zip(words, word_scores):
			print(f"{word} {score}")



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""


class main:

	def __init__(self, q, limit):
		""" use onionlandsearchengine.com

			q 		  : Query for search
			limit	  : Number of pages
		"""
		self.framework = main.framework
		self.q = self.framework.urlib(q).quote
		self.limit = 20 if limit > 20 else limit
		self._pages = ''
		self._links = []
		self.onionlandsearchengine = 'onionlandsearchengine.com'

	def run_crawl(self):
		urls = [f"https://{self.onionlandsearchengine}/search?q={self.q}&page={i}" for i in range(1, self.limit)]
		max_attempt = len(urls)
		plen = 0
		for url in range(max_attempt):
			self.framework.debug(f'[ONIONLAND] Searching in {url} page...')
			try:
				req = self.framework.request(url=urls[url])
			except:
				self.framework.print_exception()
				max_attempt -= 1
				if max_attempt == 0:
					self.framework.error('Onionland is missed!', 'util/onionland', 'run_crawl')
					break
			else:
				self._pages += req.text
				if plen == 0:
					plen = len(self.framework.reglib(self._pages).search('class="page"'))
				if plen-1 == url:
					return

	@property
	def pages(self):
		return self._pages

	@property
	def links(self):
		self._links = [x.replace('</span>\n', '') for x in self.framework.reglib(self._pages).search(r'</span>\n(https?://.*)')]
		self._links.extend([x.replace('class="link">\n', '') for x in self.framework.reglib(self._pages).search(r'class="link">\n(https?://.*)')])
		return self._links



"""
OWASP Maryam!
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import re
from bs4 import BeautifulSoup

def remove_tags(html_text):
    soup = BeautifulSoup(html_text, 'html.parser')
    return soup.get_text()

meta = {
	'name': 'Pwned database search',
	'author': 'Vikas Kundu',
	'version': '1.2',
	'description': 'Search your email for data breaches',
	'comments': (
			'Using XmlHttp API of haveibeenpwned.com',
			'to get JSON data'
	),
	'sources': ('haveibeenpwned.com',),
	'options': (
		('email', None, True, 'Email to search for breach', '-e', 'store', str),
	),
	'examples': ('email_pwned -e <email> --output',)
}


def scrap(email):
	import cloudscraper
	url = f"https://eapi.pcloud.com/checkpwned?checkemail={email}"
	scraper = cloudscraper.create_scraper()
	result = scraper.get(url)
	if result.text:
		return remove_tags(result.text)
	else:
		return False


def module_api(self):
	output = {'content':[]}
	email = self.options['email']
	self.verbose('[PAWNED] Searching for pwning...')
	pwns = scrap(email)
	if pwns:
		output['content'] = pwns
	else:
		output['content'] = 'no breach'
	self.save_gather(output, 'osint/email_pwned', email,
					 output=self.options['output'])
	return output


def module_run(self):
	output = module_api(self)
	self.alert_results(output)



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import math

class main:

	def __init__(self, query, exact, start_year, end_year, lang, extension, count):
		""" Search in the Zlibrary for books and articles
		
			query		: query to seach for
			exact		: match exact string
			start_year	: year from which to search for
			end_year	: year till which to search for
			lang		: language to search in
			extension	: file extension i.e. pdf etc.
			count		: no. of results to show
		"""
		self.framework = main.framework
		self.query = query
		self.exact = exact
		self.start_year = start_year
		self.end_year = end_year
		self.lang = lang
		self.extension = extension
		self.count = count
		self.limit = math.ceil(count/50) # No. of pages to scrap. 50 entries per page so total = ceil of limit/50
		self._books_data = ''
		self._articles_data = ''
		self.original_rand_agent_val = self.framework._global_options['rand_agent']
		self.framework._global_options['rand_agent'] = True
	
	# Adding destructor to set global option to same as before
	def __del__(self):
		self.framework._global_options['rand_agent'] = self.original_rand_agent_val
				
	def search_all(self, url, payload, source):
		scrap_result = ''
		self.framework.verbose(f'[Zlibrary Search] Fetching query in {source}...')
		cookies = {'zlib-searchView':'table'} # Cookie needed for table view instead of list
		total_entries = 0
		
		for page_no in range(1, self.limit+1):
			if 'On your request nothing has been found' in scrap_result:
				self.framework.alert('[Zlibrary Search] No data found, moving to next task...')
				return scrap_result

			self.framework.verbose(f'[Zlibrary Search] Fetching page {page_no}...')
			total_entries += 50 # 50 table entries present per page

			try:
				scrap_result += self.framework.request(f"{url}{payload}&page={page_no}" , cookies=cookies).text
			except Exception as e:
				self.framework.error(f"Zlibrary {source}, page {page_no} is missed!", 'util/zlibrary', 'search_all')
	
		return scrap_result

	def search(self):
		from bs4 import BeautifulSoup

		self.framework.verbose('[Zlibrary Search] Fetching location of servers...')
		welcome_page_url = 'https://z-lib.org/'
		
		try:
			response = self.framework.request(welcome_page_url)
		except Exception as e:
			self.framework.error(f"Zlibrary is missed!", 'util/zlibrary', 'search')
			return False
		else:	
			parsed_html = BeautifulSoup(response.text, features='lxml')

			zlib_books_url = parsed_html.body.find('span', attrs={'class': 'domain-check-domain', 'data-mode': 'books'}).text
			zlib_articles_url = parsed_html.body.find('span', attrs={'class': 'domain-check-domain', 'data-mode': 'articles'}).text

			payload_books = f"/s/{self.query}/?e={int(self.exact)}&yearFrom={self.start_year}&"+\
			f"yearTo={self.end_year}&language={self.lang}&extension={self.extension}"
			payload_articles = f"/s/{self.query}/?e={int(self.exact)}&yearFrom={self.start_year}&yearTo={self.end_year}"
			# Language and extension options not present in article search so payloads different
			
			# Appending https:// else url invalid
			self._books_data = self.search_all(f"https://{zlib_books_url}", payload_books, 'books') 
			self._articles_data = self.search_all(f"https://{zlib_articles_url}", payload_articles, 'articles')

	@property
	def books_data(self):
		return self._books_data
			
	@property
	def articles_data(self):
		return self._articles_data



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import os
import concurrent.futures
from socket import gethostbyname


meta = {
	'name': 'File Brute Force',
	'author': 'Saeed',
	'version': '1.0',
	'description': 'File/Directory brute force attack, supports cocurrency.',
	'comments': ('The wordlist option can be an url.',),
	'options': (
		('domain', None, True, 'Domain name without https?://', '-d', 'store', str),
		('count', None, False, 'Number of payloads len(max=count of payloads). default is max',
							 '-c', 'store', int),
		('wordlist', 'https://raw.githubusercontent.com/danielmiessler/SecLists/master/Discovery/Web-Content/RobotsDisallowed-Top500.txt', False, 
							'wordlist address. default is dnsnames.txt in data folder', '-w', 'store', str),
		('thread', 8, False, 'The number of links that open per round(default=8)', '-t', 'store', int),
		('wordlists', False, False, 'List of most common DNS wordlists', '-l', 'store_true', bool),
		('status_codes', '200,201,204', False, 'List of good status codes(default="200,201,204")', '-s', 'store', str),
		('redirect', True, False, 'Allow redirection(default=True)', '-r', 'store_true', bool),
	),
	'examples': ('filebrute -d <DOMAIN> --output',
				'filebrute -d <DOMAIN> -w <WORDLIST>')
}

URLS = []

LISTS = {"https://raw.githubusercontent.com/danielmiessler/SecLists/master/Discovery/Web-Content/CMS/wp-plugins.fuzz.txt": 'small',
		 "https://raw.githubusercontent.com/danielmiessler/SecLists/master/Discovery/Web-Content/CMS/wp-themes.fuzz.txt": 'small',
		 "https://raw.githubusercontent.com/danielmiessler/SecLists/master/Discovery/Web-Content/CMS/wordpress.fuzz.txt": 'small',
		 "https://raw.githubusercontent.com/danielmiessler/SecLists/master/Discovery/Web-Content/tests.txt": 'small',
		 "https://raw.githubusercontent.com/danielmiessler/SecLists/master/Discovery/Web-Content/jboss.txt": 'small',
		 "https://raw.githubusercontent.com/danielmiessler/SecLists/master/Discovery/Web-Content/PHP.fuzz.txt": 'small',
		 "https://raw.githubusercontent.com/danielmiessler/SecLists/master/Discovery/Web-Content/IIS.fuzz.txt": 'small',
		 "https://raw.githubusercontent.com/danielmiessler/SecLists/master/Discovery/Web-Content/RobotsDisallowed-Top10.txt": 'small',
		 "https://raw.githubusercontent.com/danielmiessler/SecLists/master/Discovery/Web-Content/RobotsDisallowed-Top100.txt": 'small',
		 "https://raw.githubusercontent.com/danielmiessler/SecLists/master/Discovery/Web-Content/RobotsDisallowed-Top500.txt": 'small',
		 "https://raw.githubusercontent.com/danielmiessler/SecLists/master/Discovery/Web-Content/RobotsDisallowed-Top1000.txt": 'small',
		 "https://raw.githubusercontent.com/danielmiessler/SecLists/master/Discovery/Web-Content/CMS/sitemap-magento.txt": 'medium',
		 "https://raw.githubusercontent.com/danielmiessler/SecLists/master/Discovery/Web-Content/LinuxFileList.txt": 'large',
		 "https://raw.githubusercontent.com/danielmiessler/SecLists/master/Discovery/Web-Content/SVNDigger/all.txt": 'large',
}
def thread(self, function, hostname, wordlist, thread_count, status_codes):
	threadpool = concurrent.futures.ThreadPoolExecutor(max_workers=thread_count)
	futures = (threadpool.submit(function, self, hostname, word, status_codes) for word in wordlist if not word.startswith("#"))
	counter = 1
	for _ in concurrent.futures.as_completed(futures):
		print(f"Checking payload {counter}, hits: {len(URLS)}", end='\r')
		counter += 1
	print(os.linesep)

def request_checker(self, hostname, word, status_codes):
	hostname_join = self.urlib(hostname).join(word)
	try:
		req = self.request(hostname_join, allow_redirects=self.options['redirect'])
	except Exception as e:
		if 'timed out' in str(e.args):
			self.verbose(f"{hostname_join} => Timed out", 'O')
		else:
			self.debug(f"{list(e.args)[0]}:'{hostname_join}'", 'O')
	else:
		if 'Location' in req.headers:
			location = req.headers['Location']
			join = self.urlib(hostname).join(location)
			try:
				req = self.request(join)
			except:
				return
		if req.status_code in status_codes:
			URLS.append(hostname_join)
			self.output(f"{hostname_join}{' '*50}", 'G')

def remote_list(self, addr):
	req = self.request(addr)
	headers = req.headers
	keys = list(map(str.lower, list(headers.keys())))
	cond1 = 'content-type' in keys and 'text/plain' in str(headers).lower()
	cond2 = 'githubusercontent.com' in addr.lower()
	if cond1 or cond2:
		wordlist = req.text.split('\n')
		if '' in wordlist:
			wordlist.pop(wordlist.index(''))
		return wordlist
	else:
		self.error(f"{addr} value is not text/plain", 'filebrute', 'remote_list')
	return []

def module_api(self):
	if self.options["wordlists"]:
		return {'lists': LISTS}

	domain = self.options['domain']
	wordlist = self.options['wordlist']
	if not domain or not wordlist:
		return {'error': 'Arguments are incorrect'}
	urlib = self.urlib(domain)
	domain = urlib.sub_service('http')
	domain = domain.replace('www.', '')
	status_codes = list(map(int, self.options['status_codes'].split(',')))
	count = self.options['count']
	if '://' in wordlist:
		dlist = remote_list(self, wordlist)
	else:
		dlist = self._is_readable(wordlist)
		if dlist:
			dlist = dlist.read().split()
		else:
			dlist = []

	dlen = len(dlist)
	if not count:
		count = dlen
	else:
		count = dlen if count > dlen else count

	self.heading(f"Starting File/Dir brute force with {count} payload", level=0)
	thread(self, request_checker, domain, dlist[:count], self.options['thread'], status_codes)
	output = {'urls': URLS}
	self.save_gather(output, 'footprint/filebrute', domain, output=self.options['output'])
	return output

def module_run(self):
	if self.options["wordlists"]:
		self.alert('common DNS wordlists')
		self.table(LISTS.items(), header=('list', 'scale'))
		return
	self.alert_results(module_api(self))



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import re

meta = {
	'name': 'Reddit Search',
	'author': 'Kunal Khandelwal',
	'version': '1.0',
	'description': 'Search your query in the Reddit and show the results.',
	'sources': ('google', 'bing', 'metacrawler', 'millionshort', 'etools', 'duckduckgo'),
	'options': (
		('query', None, True, 'Query string', '-q', 'store', str),
		('limit', 1, False, 'Search limit(number of pages, default=1)', '-l', 'store', int),
		('count', 50, False, 'Number of links per page(min=10, max=100, default=50)', '-c', 'store', int),
		('engine', 'google', False, 'Engine names for search(default=google)', '-e', 'store', str),
		('thread', 2, False, 'The number of engine that run per round(default=2)', '-t', 'store', int),
		('search-type', 'posts', False, 'Type of search i.e. posts or comments(default=posts)', '-s', 'store', str),
		('username', '', False, 'Search data of a particular username', '-u', 'store', str),
		('subreddit', '', False, 'Search data in a particular subreddit', '-r', 'store', str),
		('before', '', False, 'Search data before this date(format=d/m/y H:M:S) i.e. 7/6/21 12:0:0', '-b', 'store', str),
		('after', '', False, 'Search data after this date(format=d/m/y H:M:S) i.e. 7/6/21 12:0:0', '-a', 'store', str),
		('score', '', False, 'Score of the data to search i.e. >1 or <2 etc.', '-o', 'store', str),
	),
	'examples': ('reddit -q <QUERY> -l 15 --output',)
}

LINKS = []
PAGES = ''
USERNAMES = []

def search(self, name, q, q_formats, limit, count, search_type, username, subreddit, before, after, score):
	global PAGES,LINKS,USERNAMES
	engine = getattr(self, name)
	q = q_formats[f"{name}_q"] if f"{name}_q" in q_formats else q_formats['default_q']
	varnames = engine.__init__.__code__.co_varnames
	if 'limit' in varnames and 'count' in varnames:
		attr = engine(q, limit, count)
	elif 'limit' in varnames:
		attr = engine(q, limit)
	elif name == 'reddit_pushshift':
		attr = engine(q, count, search_type, username, subreddit, before, after, score)	
	else:
		attr = engine(q)
	
	# Some engines like reddit_pushshift return direct usernames so to avoid errors in other engines, check first
	attr.run_crawl()
	LINKS += attr.links if hasattr(attr, 'links') else []
	PAGES += attr.pages if hasattr(attr, 'pages') else ''
	USERNAMES += attr.usernames if hasattr(attr, 'usernames') else []


def module_api(self):
	query = self.options['query']
	limit = self.options['limit']
	count = self.options['count']
	engine = self.options['engine'].split(',')
	search_type = self.options['search_type']
	username = self.options['username']
	subreddit = self.options['subreddit']
	before = self.options['before']
	after = self.options['after']
	score = self.options['score']

	output = {'links': [], 'usernames': []}
	q_formats = {
		'default_q': f"site:www.reddit.com {query}",
		'millionshort_q': f'site:www.reddit.com "{query}"',
		'qwant_q': f'site:www.reddit.com {query}',
		'reddit_pushshift_q': query
	}
	
	self.thread(search, self.options['thread'], engine, query, q_formats, limit, count, search_type, username, subreddit, before, after, score, meta['sources'])

	links = list(self.reglib().filter(r"https?://(www\.)?reddit\.com/", list(set(LINKS))))

	for i in USERNAMES: # For classes that return direct usernames
		output['usernames'].append(i)

	usernames = self.page_parse(PAGES).get_networks
	for user in list(set(usernames.get('Reddit'))):
		user = user[(user.rfind('/'))+1:]
		if user not in output['usernames']:
			output['usernames'].append(user)
	
	for link in links:
		if re.search(r"reddit\.com/r/", link) and "/about/" not in link:
			post_url = re.sub(r"https?://(www\.)?reddit\.com/r/", '', link)
			post_url = post_url.rsplit('/')
			subreddit = post_url[0]
			try:
				post = post_url[3]
			except Exception as e:
				continue
			post = post.replace('_', ' ')
			post = self.urlib(post).unquote
			output['links'].append([link, f"{post.title()} => r/{subreddit}"])
			

	self.save_gather(output,
					 'search/reddit', query, output=self.options.get('output'))
	return output

def module_run(self):
	output = module_api(self)
	self.alert('usernames')
	for user in output['usernames']:
		self.output(f"\t{user}", 'G')
	self.alert('links')
	for item in output['links']:
		link,title = item[0],item[1]
		self.output(f"\t{title}")
		self.output(f"\t\t{link}", 'G')



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import re
import concurrent.futures
import requests

meta = {
	'name': 'Domain Reputation',
	'author': 'Kunal Khandelwal',
	'version': '0.3',
	'description': 'Check domain reputation with different sources and provide a summary of combined results.',
	'sources': ('barracudacentral', 'mxtoolbox', 'multirbl', 'norton'),
	'options': (
		('domain', None, True, 'Domain name without https?://', '-d', 'store', str),
		('engines', 'barracudacentral,multirbl,mxtoolbox,norton', False, 'Search engine names(default=[barracudacentral,'
					'multirbl, mxtoolbox])'
		 , '-e', 'store', str),
		('thread', 2, False, 'The number of engine that run per round(default=2)', '-t', 'store', int),
	),
	'examples': ('domain_reputation -d example.com',
				 'domain_reputation -d 93.184.216.34'
				 )
}

BLACKLIST = []
OUTPUT = {}
LISTS = 0
RESULT = ''
OUTPUT = {'category': '', 'number': 0, 'blacklists': [], 'absence': 0, 'norton': ''}

def thread(self, function, thread_count, engines, q, sources):
	threadpool = concurrent.futures.ThreadPoolExecutor(max_workers=thread_count)
	futures = (threadpool.submit(
		function, self, name, q) for name in engines if name in sources)
	for _ in concurrent.futures.as_completed(futures):
		pass

def search(self, name, q):
	engine = eval(name)
	engine(self, q)

def norton(self, q):
	global RESULT
	self.verbose('[NORTON] Scanning domain...')
	try:
		req = self.request(f'https://safeweb.norton.com/report/show?url={q}')
	except Exception as e:
		self.error(f"Norton could not scan! due to {e}", 'domain_reputation', 'norton')
	else:
		e_reg = 'is a known dangerous web page'
		s_reg = 'found no issues with'
		if e_reg in req.text:
			RESULT = f"Threat Report: {q} {e_reg}"
		elif s_reg in req.text:
			RESULT = f"Threat Report: {s_reg} {q}"
		else:
			RESULT = f"Threat Report: {q} not yet rated as dangerous."

def barracudacentral(self, q):
	global LISTS,OUTPUT
	self.verbose('[Barracuda] Searching in barracuda...')
	data = {
		'lookup_entry': q,
		'submit': 'Check Reputation'
	}
	try:
		req = self.request('https://www.barracudacentral.org/lookups/lookup-reputation', method='POST',
						   data=data)
	except Exception as e:
		self.error('Barracuda is missed!', 'domain_reputation', 'barracudacentral')
	else:
		reg = re.compile(r"categories: <strong>([\w\d\s-]+)")
		if 'listed as "poor"' not in req.text:
			category = reg.findall(req.text)[0]
			OUTPUT['category'] = category
		else:
			BLACKLIST.append('BARRACUDA')
		LISTS += 1

def mxtoolbox(self, q):
	global LISTS
	self.verbose('[MXTOOLBOX] Searching in mxtoolbox...')
	try:
		headers = {'TempAuthorization': 'ca1e3b2f-cd6c-4886-9446-88f4ff7a960d',
				   'Referer': f'https://mxtoolbox.com/SuperTool.aspx?action=blacklist:{q}&run=toolpage',
				   'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_5) AppleWebKit/537.36 '
								 '(KHTML, like Gecko) Chrome/50.0.2661.102 Safari/537.36'
				   }
		req = requests.get(f'http://mxtoolbox.com/api/v1/Lookup?command=blacklist&argument={q}&resultIndex=1'
						   f'&disableRhsbl=true&format=2', headers=headers)
		j = req.json()
		j = j['HTML_Value']
		num_reg = re.compile(r'<strong>([\d]+)</strong>')
		num_lists = num_reg.findall(j)[0]
		list_reg = re.compile('<tr>(.+?)</tr>')
		blacklist_reg = re.compile(r'<span class="bld_name">([\d\w\s]+)</span>')
		lists = list_reg.findall(j)[:-1]
	except Exception as e:
		if 'API' in req.text:
			self.error('Mxtoolbox needs a valid API key', 'domain_reputation', 'mxtoolbox')
		else:
			self.error('Mxtoolbox is missed!', 'domain_reputation', 'mxtoolbox')
	else:
		for blacklist in lists:
			if 'LISTED' in blacklist:
				try:
					list_name = blacklist_reg.findall(blacklist)[0]
				except:
					pass
				BLACKLIST.append(list_name)
		LISTS += int(num_lists)

def multirbl(self, q):
	global LISTS
	self.verbose('[Multirbl] Searching in multirbl.valli.org...')
	try:
		req = self.request(f'http://multirbl.valli.org/lookup/{q}.html', verify=False)
		text = req.text
		hash_reg = re.compile(r'"asessionHash": "([\w]+)"')
		l_id_hash = re.compile(r'<td class="l_id">([\d]+)')
		hash = hash_reg.findall(text)[0]
		l_ids = l_id_hash.findall(text)
		num_test_reg = re.compile(r'nof: ([\d]+),')
		num_test = num_test_reg.findall(text)[0]
	except Exception as e:
		self.error('Multirbl is missed', 'domain_reputation', 'multirbl')
	else:
		thread_pool = concurrent.futures.ThreadPoolExecutor(max_workers=2)
		futures = (thread_pool.submit(__multirbl, self, l_id, hash, l_ids, q) for l_id in range(int(num_test)))
		for _ in concurrent.futures.as_completed(futures):
			pass
		LISTS += int(num_test)

def __multirbl(self, l_id, hash, l_ids, q):
	try:
		data = {'ash': hash,
				'rid': f'DNSBLBlacklistTest_{l_id}',
				'q': q,
				'lid': l_ids[l_id],
				}
		req = self.request(f'http://multirbl.valli.org/json-lookup.php', method='POST',
						   verify=False, data=data)
		j = req.json()
		if not j['result']:
			BLACKLIST.append(j['name'])
	except Exception as e:
		pass

def module_api(self):
	global LISTS
	query = self.options['domain']
	engines = self.options['engines'].split(',')
	thread(self, search, self.options['thread'], engines, query, meta['sources'])
	OUTPUT['number'] = len(list(set(BLACKLIST)))
	OUTPUT['blacklists'] = list(set(BLACKLIST))
	if LISTS != 0:
		presence = len(list(set(BLACKLIST))) / LISTS
		OUTPUT['absence'] = 100 - (presence*100)
	OUTPUT['norton'] = RESULT
	return OUTPUT

def module_run(self):
	output = module_api(self)
	self.output(f"CATEGORY\n\t{output['category']}")
	self.output(f"NUMBER OF PRESENCE ON BLACKLIST\n\t{output['number']}")
	self.output('BLACKLISTS')
	for i in output['blacklists']:
		self.output(i, 'B')
	self.output(f"ABSENCE PERCENTAGE\n\t{output['absence']}")
	self.output(f"NORTON SAFE WEB REPORT\n\t{output['norton']}")



"""
OWASP Maryam!
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from json import dumps
from datetime import datetime

meta = {
	'name': 'Tweet Search',
	'author': 'Kaushik',
	'version': '0.7',
	'description': 'Search tweets from twitter',
	'required': ('$iris',),
	'options': (
		('query', None, True, 'Query string', '-q', 'store', str),
		('limit', 15, False, 'Max result count (default=15)', '-l', 'store', int),
		('sentiment', False, False, 'Do sentiment Analysis on tweets(default=False)', '-s', 'store_true', bool),
		('daterange', -1, False, 'Number of past days to fetch tweets for', '-d', 'store', int),
		('monthly', False, False, 'Group datewise sentiment values by month', '-m', 'store_true', bool),
		('verbose', False, False, 'Print all tweet details as json', '-v', 'store_true', bool),
	),
	'examples': ('tweet_search -q <QUERY> -l 15',
		'tweet_search -q <QUERY> -l 15 --sentiment --daterange 2')
}

SHOW_MSG = {'neg': 'negative', 'neu': 'neutral', 'pos': 'positive', 'compound': 'compound'}

def module_api(self):
	query = self.options['query']
	limit = self.options['limit']
	sentiment = self.options['sentiment']
	daterange = self.options['daterange']
	monthly = self.options['monthly']

	if sentiment:
		verbose = False
	else:
		verbose = self.options['verbose']

	run = self.tweet_search(query, limit, verbose, daterange)
	run.run_crawl()

	output = {'results': run.tweets['all'] if daterange == -1 else run.tweets}
	if sentiment:
		if 'sentiment' not in self._loaded_modules:
			self._load_modules('iris')
		output['sentiment'] = {}
		if monthly:
			allmonths = {}
		sent_mod = self._loaded_modules['sentiment']
		self.options['pipe'] = True
		self.options['thread'] = 1
		if daterange == -1:
			sent_mod.DATA = output['results']
			output['sentiment'] = sent_mod.module_api(self)
		else:
			for day in output['results']:
				sent_mod.DATA = output['results'][day]
				output['sentiment'][day] = sent_mod.module_api(self)
				if monthly:
					month = datetime.strptime(day, '%Y-%m-%d').strftime('%B')
					curr_day = output['sentiment'][day]

					if month not in allmonths:
						allmonths[month] = {}
						allmonths[month]['maxes'] = curr_day['maxes']
						allmonths[month]['overall'] = curr_day['overall']
						allmonths[month]['count'] = 1

					else:
						for key in SHOW_MSG:
							if allmonths[month]['maxes'][key][0] < curr_day['maxes'][key][0]:
								allmonths[month]['maxes'][key] = curr_day['maxes'][key]
						value = allmonths[month]['overall'][key]
						count = allmonths[month]['count']
						allmonths[month]['count'] += 1
						allmonths[month]['overall'][key] = (value*(count)+curr_day['overall'][key])/count+1

			if monthly:
				output['sentiment']['monthly'] = allmonths

	self.save_gather(output, 'osint/twitter', query, output=self.options['output'])
	return output

	

def module_run(self):
	sentiment = self.options['sentiment']
	daterange = self.options['daterange']
	monthly = self.options['monthly']
	output = module_api(self)

	def print_sentiment(sent):
		for i in sent['overall']:
			self.alert(SHOW_MSG[i])
			self.output(f"\t{sent['overall'][i]}")
			if sent['maxes'][i]:
				self.output(f"\tmost {SHOW_MSG[i]} sentence:  {sent['maxes'][i][1]}")

	if not self.options['verbose']:
		if daterange != -1:
			for day in output['results']:
				print()
				self.output(day)
				for tweet in output['results'][day]:
					self.output(tweet)
				if sentiment:
					print()
					self.alert('Sentiment')
					sent = output['sentiment'][day]
					print_sentiment(sent)
			if sentiment and monthly:
				print()
				self.output('Monthwise Sentiment Aggregate')
				allmonths = output['sentiment']['monthly']
				for month in allmonths:
					self.alert(f"Month: {month}")
					print_sentiment(allmonths[month])
					print()

		else:
			for tweet in output['results']:
				self.output(tweet)

			if sentiment:
				print()
				sent = output['sentiment']
				print_sentiment(sent)
	else:
		self.output(dumps(output, indent=4))



"""
OWASP Maryam!
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from lxml import html
from json import loads

class main:
	def __init__(self, q, limit=3, count=28):
		""" bing.com search engine
			q     : Query for search
			limit : Number of pages
			count : Number of results
		"""
		self.framework = main.framework
		self.q = q
		self.agent = 'Mozilla/5.0 (X11; Linux x86_64; rv:100.0) Gecko/20100101 Firefox/100.0'
		self.url = 'https://www.bing.com/images/search'
		self._pages = ''
		self.count = 28 if count > 28 else count
		self.limit = 10 if limit > 10 else limit

	def run_crawl(self):
		set_page = lambda x: (x*self.count)+1
		payload = {'count': self.count, 'first': 1, 'form': 'IBASEP', 'q': self.q}
		max_attempt = 0
		for i in range(self.limit):
			self.framework.verbose(f"[BING Image] Searching in {i} page...", end='\r')
			try:
				req = self.framework.request(
					url=self.url,
					params=payload,
					headers={'user-agent': self.agent},
					allow_redirects=True)
			except Exception as e:
				self.framework.error(f"ConnectionError: {e}", 'util/engines/bing_images', 'run_crawl')
				max_attempt += 1
				if max_attempt == self.limit:
					self.framework.error('Bing is missed!', 'util/engines/bing_images', 'run_crawl')
					break
			else:
				self._pages += req.text
				payload['first'] = set_page(i+1)

	@property
	def results(self):
		items = []
		tree = html.fromstring(self._pages)
		for item in tree.xpath('//div[@class="imgpt"]'):
			try:
				info = item.xpath('.//div[@class="img_info hon"]')[0].xpath("./span[@class='nowrap']")[0].text
				m = loads(item.xpath('./a/@m')[0])
				title = m.get('t', '').replace(u'\ue000', '').replace(u'\ue001', '')
				items.append({'i': m['murl'],
								't': title,
								'a': m['purl'],
								'd': info
								})
			except:
				continue
		return items

	@property
	def pages(self):
		return self._pages

	@property
	def links(self):
		return [x['a'] for x in self.results]




"""
OWASP Maryam!
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import re
import json
from socket import gethostbyaddr

meta = {
	'name': 'DNS Searcher',
	'author': 'Saeed',
	'version': '2.5',
	'description': 'Search in the open-sources to find subdomans.',
	'sources': ('securitytrails', 'bing', 'google', 'metacrawler', 'startpage',
				'netcraft', 'threatcrowd', 'virustotal', 'otx', 'carrot2', 'crt',
				'threatminer', 'jldc', 'bufferover', 'rapiddns', 'certspotter',
				'sublist3r', 'riddler', 'sitedossier', 'duckduckgo', 'dnsdumpster', 'yougetsignal', 'pastebin',
				'urlscan'),
	'options': (
		('domain', None,
		 True, 'Domain name without https?://', '-d', 'store', str),
		('limit', 3, False, 'Search limit(number of pages, default=3)', '-l', 'store', int),
		('count', 30, False, 'number of results per page(min=10, max=100, default=30)', '-c', 'store', int),
		('engines', 'otx,securitytrails', False, 'Search engine names. e.g bing,google,...[otx by default]', '-e', 'store', str),
		('thread', 2, False, 'The number of engine that run per round(default=2)', '-t', 'store', int),
		('max', False, False, 'Using all of sources(max limit=15, max count=50)', '--max', 'store_true', bool),
		('validate', False, False, 'Validate the domains(Remove dead subdomains) \
			found and display their IP(default=False)', '--validate', 'store_true', bool),
		('silent', False, False, 'Output without any color and message([Warn]This sets the \
			value of verbosity to zero!, default=False)', '--silent', 'store_true', bool),
		('reverse', None, False, 'Use reverse DNS search. \
			Input could be a list of ip addresses(with comma separator)\
			 or could be a filename that contains ip addresses', '-r', 'store', str),
	),
	'examples': ('dns_search -d example.com --output', 'dns_search -d example.com -e\
		google,bing,yahoo -l 3 -t 3 --output', 
				 'dns_search -d example.com --max --validate --silent --output',
				 'dns_search --reverse 1.1.1.1,2.2.2.2,3.3.3.3',
				 'dns_search --reverse ips.txt')
}

HOSTNAMES = []

def set_data(lst):
	global HOSTNAMES
	for sub in lst:
		if '%' in sub:
			sub = sub.split('%')[0]
		if sub.startswith('.'):
			sub = sub[1:]
		if sub.count('.') < 2:
			continue
		HOSTNAMES.append(sub)

def search(self, name, q, q_format, limit, count):
	inside_func = 0
	if hasattr(self, name):
		engine = getattr(self, name)
		varnames = engine.__init__.__code__.co_varnames
	else:
		inside_func = 1
		engine = eval(name)
		varnames = engine.__code__.co_varnames
	if 'limit' in varnames and 'count' in varnames:
		if not inside_func:
			attr = engine(q, limit, count)
		else:
			attr = engine(self, q, limit, count)
	elif 'limit' in varnames:
		if not inside_func:
			attr = engine(q, limit)
		else:
			attr = engine(q, limit)
	else:
		if not inside_func:
			attr = engine(q)
		else:
			attr = engine(self, q)
	if not inside_func:
		attr.run_crawl()
		set_data(attr.dns)

def riddler(self, q):
	self.verbose('[RIDDLER] Searching in riddler...')
	try:
		req = self.request(
			f"https://riddler.io/search?q=pld:{q}&view_type=data_table")
	except Exception as e:
		self.error('Riddler is missed!', 'dns_search', 'riddler')
	else:
		j = self.page_parse(req.text).get_dns(q) or []
		set_data(j)

def sitedossier(self, q):
	self.verbose('[SITEDOSSIER] Searching in reddler...')
	next_page = ''
	while True:
		try:
			req = self.request(
				f"http://www.sitedossier.com/parentdomain/{q}/{next_page}")
		except Exception as e:
			self.error('Sitedossier is missed!', 'dns_search', 'sitedossier')
			break
		else:
			text = req.text
			if 'We apologise for any inconvenience this may cause.' in text:
				self.verbose('[SITEDOSSIER] Unusual Activity!')
				break
			next_page = re.search(r'<a href="/parentdomain/[\w\.\-]+/([\d]+)">', req.text)
			if next_page:
				next_page = next_page.group(1)
			j = self.page_parse(req.text).get_dns(q) or []
			set_data(j)
			if next_page == None:
				break
			self.output(f"[SITEDOSSIER] Searching in the {next_page} page...")

def threatcrowd(self, q):
	global HOSTNAMES
	self.verbose('[THREATCROWD] Searching in threatcrowd...')
	try:
		req = self.request(
			'https://threatcrowd.org/searchApi/v2/domain/report/?domain=' + q)
	except Exception as e:
		self.error('ThreatCrowd is missed!', 'dns_search', 'threatcrowd')
	else:
		txt = re.sub(r'[\t\n ]+', '', req.text)
		txt = re.findall(
			r'"subdomains":(\[["\.A-z0-9_\-,]+\])', txt)
		hosts = list(set(txt[0][1:-1].split(',')))
		for host in hosts:
			host = host[1:-1]
			if host.startswith('.'):
				host = host[1:]
			HOSTNAMES.append(host)

def threatminer(self, q):
	self.verbose('[THREATMINER] Searching in threatminer...')
	try:
		req = self.request(
			f"https://api.threatminer.org/v2/domain.php?q={q}&rt=5")
	except Exception as e:
		self.error('ThreatMiner is missed!', 'dns_search', 'threatminer')
	else:
		j = req.json()['results'] or []
		set_data(j)

def rapiddns(self, q):
	self.verbose('[RAPIDDNS] Searching in rapiddns...')
	try:
		req = self.request(
				f"https://rapiddns.io/subdomain/{q}?full=1")
	except Exception as e:
		self.error('Rapiddns is missed!', 'dns_search', 'rapiddns')
	else:
		j = self.page_parse(req.text).get_dns(q) or []
		set_data(j)

def dnsdumpster(self, q):
	self.verbose('[DNSDUMPSTER] Searching in dnsdumpster...')
	init_res = self.request('https://dnsdumpster.com/', method='GET')  # initial response from dnsdumpster
	set_cookie = init_res.headers.get('Set-Cookie')  # getting crsftoken from cookie field
	cookie = set_cookie[:set_cookie.index(';') + 1]
	headers = {
		'authority': 'dnsdumpster.com',
		'origin': 'https://dnsdumpster.com',
		'content-type': 'application/x-www-form-urlencoded',
		'sec-fetch-dest': 'document',
		'referer': 'https://dnsdumpster.com/',
		'cookie': cookie
	}
	try:
		# using regex to search for the csrfmiddlewaretoken (64 characters)
		i = re.search(r'name="csrfmiddlewaretoken" value="', init_res.text).end()
		csrf_token = init_res.text[i:i + 64]
		data = {'csrfmiddlewaretoken': csrf_token, 'targetip': q}
		req = self.request('https://dnsdumpster.com/', method='POST', headers=headers, data=data)
	except Exception:
		self.error('DNSdumpster is missed!', 'dns_search', 'dnsdumpster')
	else:
		j = self.page_parse(req.text).get_dns(q) or []
		set_data(j)

def yougetsignal(self, q):
	self.verbose('[YOUGETSIGNAL] Searching in yougetsignal...')
	headers = {
		'authority': 'domains.yougetsignal.com',
		'accept': 'text/javascript, text/html, application/xml, text/xml, */*',
		'origin': 'https://www.yougetsignal.com',
		'referer': 'https://www.yougetsignal.com/',
	}
	data = {'remoteAddress': q}
	try:
		req = self.request('https://domains.yougetsignal.com/domains.php', method='POST', headers=headers, data=data).json()
	except Exception as e:
		self.error('YouGetSignal is missed!', 'dns_search', 'yougetsignal')
	else:
		j = [x[0] if len(x)>1 else '' for x in req.get('domainArray')]
		set_data(j)

def certspotter(self, q):
	self.verbose('[CERTSPOTTER] Searching in certspotter...')
	try:
		req = self.request(
			f"https://api.certspotter.com/v1/issuances?domain={q}&include_subdomains=true&expand=dns_names")
	except Exception as e:
		self.error('CERTSPOTTER is missed!', 'dns_search', 'certspotter')
	else:
		text = req.text
		if "rate_limit" in text:
			self.error('Too many request please try again later', 'dns_search', 'certspotter')
			return "certspotter"
		j = self.page_parse(req.text).get_dns(q) or []
		set_data(j)

def sublist3r(self, q):
	global HOSTNAMES
	self.verbose('[SUBLIST3R] Searching in sublist3r...')
	try:
		req = self.request(
			'https://api.sublist3r.com/search.php?domain=' + q)
	except Exception as e:
		self.error('SUBLIST3R is missed!', 'dns_search', 'sublist3r')
	else:
		j = json.loads(req.text)
		set_data(j)

def jldc(self, q):
	self.verbose('[JLDC] Searching in jldc.me...')
	try:
		req = self.request(
			f"https://jldc.me/anubis/subdomains/{q}")
	except Exception as e:
		self.error('JLDC is missed!', 'dns_search', 'jldc')
	else:
		if 'Too many request' in req.text:
			self.error('Too many request please try again later', 'dns_search', 'jldc')
			return 'jldc'
		j = list(req.json()) or []
		set_data(j)

def bufferover(self, q):
	self.verbose('[BUFFEROVER] Searching in bufferover.run...')
	try:
		req = self.request(
			f'https://dns.bufferover.run/dns?q=.{q}')
	except Exception as e:
		self.error('BufferOver is missed!', 'dns_search', 'bufferover')
	else:
		j = list(req.json()['FDNS_A']) or []
		set_data([(x.split(',')[1] if len(x.split(',')) == 2 else x) for x in j])

def otx(self, q):
	self.verbose('[OTX] Searching in otx.alienvault...')
	try:
		req = self.request(
			f"https://otx.alienvault.com/api/v1/indicators/domain/{q}/passive_dns")
	except Exception as e:
		self.error('OTX is missed!', 'dns_search', 'otx')
	else:
		j = self.page_parse(req.text).get_dns(q)
		set_data(j)

def securitytrails(self, q):
	sectrail_url = 'securitytrails.com'
	enu_url = f"https://{sectrail_url}/list/apex_domain/{q}"
	trim = ['<script id="__NEXT_DATA__" type="application/json">','</script><script nomodule=""']
	domains = []
	self.verbose('[SECTRAILS] Enumerating dns records...')
	try:
		req = self.request(enu_url)
	except Exception as e:
		self.error('Sectrails is missed!', 'dns_search', 'securitytrails')
	else:
		result = req.text.split(trim[0])[1].split(trim[1])[0]
		req_json = json.loads(result)
		for x in req_json['props']['pageProps']['apexDomainData']['data']['records']:
			domains.append(x['hostname'])
		set_data(domains)

def reverse_dns(self, ips):
	reg_ip = self.reglib().ip_m
	if not re.search(reg_ip, ips):
		r = self._is_readable(ips)
		if not r:
			self.output("Invalid filename: " + ips)
			return
		ips = [x.replace('\n', '') for x in r.readlines() if re.search(reg_ip, x)]
	else:
		ips = ips.split(',')

	data = []
	titles = ['hostnames', 'ip_address_list', 'alias_list']
	for i, ip in enumerate(ips):
		tire = ()
		try:
			hostname, alias_list, addr_list = gethostbyaddr(ip)
			tire += (hostname,)
			tire += (','.join(addr_list),)
			tire += (','.join(alias_list),) if alias_list else ('-',)
			data.append(tire)
		except: 
			self.output('Invalid IP address: ' + ip)
			continue
	self.table(data, header=titles, title='REVERSE DNS')

def module_api(self):
	domain = self.options['domain']
	reverse = self.options['reverse']
	if reverse:
		self.reverse_dns(reverse)
		return
	if not domain:
		self.output('Domain name has not been set.', 'R')
		return
	query = domain
	domain_attr = self.urlib(domain)
	domain = domain_attr.sub_service('http')
	domain_name = self.urlib(domain).netloc
	MAX = self.options['max']
	limit = 15 if MAX else self.options['limit']
	count = 50 if MAX else self.options['count']
	engines = self.options['engines']
	verb = self._global_options['verbosity']
	output = {'hostnames': []}
	if self.options['silent']:
		self._global_options['verbosity'] = 0
	if engines == None:
		engines = 'otx,duckduckgo'
	engines = meta['sources'] if MAX else self.options['engines'].lower().split(',')
	self.thread(search, self.options['thread'], engines, query, {}, limit, count, meta['sources'])
	output['hostnames'] = list(set(HOSTNAMES))
	if self.options['validate']:
		validate_hosts = []
		for sub in HOSTNAMES:
			sub = re.sub(r'^\*\.', '', sub)
			if re.search(r'[^\w\d\.\-]+', sub):
				continue
			try:
				ip = self.urlib("http://" + sub).ip
			except Exception as e:
				pass
			else:
				sub = f"{sub} {ip}"
				validate_hosts.append(sub)
		output['hostnames'] = list(set(validate_hosts))

	self._global_options['verbosity'] = verb
	self.save_gather(output, 'osint/dns_search', domain_name, output=self.options['output'])
	return output

def module_run(self):
	self.alert_results(module_api(self))



"""
OWASP Maryam!
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import re
import concurrent.futures

class main:
	def __init__(self, q, limit=1, count=10):
		""" pastebin link
		q       : The query
		limit   : Number of pages
		"""
		self.framework = main.framework
		self.q = q.split('_')[0]
		self._pages = ''
		self.sources = ['google', 'bing', 'etools', 'duckduckgo', 'millionshort', 'qwant']
		self._domains = []
		self.extract_url = 'https://pastebin.com/raw'
		self.limit = limit
		self.count = count
		self.thread = 3
		self._links = []
		self._links_and_titles = []
		self.q_formats = {
			'default_q': f'site:pastebin.com "{self.q}"',
			'qwant_q': f'site:pastebin.com {self.q}'
		}

	def search(self, self2, name, q, q_formats, limit, count):
		engine = getattr(self.framework, name)
		q = self.q_formats[f"{name}_q"] if f"{name}_q" in self.q_formats else self.q_formats['default_q']
		varnames = engine.__init__.__code__.co_varnames
		if 'limit' in varnames and 'count' in varnames:
			attr = engine(q, limit, count)
		elif 'limit' in varnames:
			attr = engine(q, limit)
		else:
			attr = engine(q)

		attr.run_crawl()
		self._pages += attr.pages
		self._pastebin_pages = ''

	# def open_pages(self, link):
	# 	heading = re.search(r"pastebin\.com/([\w\d]+)", link)
	# 	title = 'no title'
	# 	if heading:
	# 		head_raw = f"https://pastebin.com/raw/{heading.group(1)}"
	# 		print(head_raw)
	# 		# try:
	# 		head_req = self.framework.request(url=head_raw).text
	# 		# except Exception as e:
	# 			# self.framework.verbose('Pastebin is missed!')
	# 		# else:
	# 		head_title = f"{self.q} pastes {head_req.splitlines()[0].lstrip()[:30]}...".ljust(10, ' ')
	# 		title = head_title.title()
	# 		self._pastebin_pages += head_req
	# 		self._links_and_titles.append([link, title])

	def run_crawl(self):
		self.framework.thread(self.search, self.thread, self.sources, self.q, self.q_formats, self.limit, self.count, self.sources)
		self._links = list(set(self.framework.reglib(self._pages).search(r"https://pastebin\.com/[\w\d]+")))
		# print(links)
		# self.framework.verbose('Rearranging paste links [give it a few seconds]...')
		# with concurrent.futures.ThreadPoolExecutor(max_workers=10) as executor:
			# [executor.submit(self.open_pages, url) for url in links]

	@property
	def pages(self):
		return self._pastebin_pages

	@property
	def links(self):
		return self._links

	@property
	def links_and_titles(self):
		return self._links_and_titles

	@property
	def dns(self):
		return self.framework.page_parse(self._pastebin_pages).get_dns(self.q)

	@property
	def emails(self):
		return self.framework.page_parse(self._pastebin_pages).get_emails(self.q)



"""
OWASP Maryam!
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import concurrent.futures

meta = {
	'name': 'Iris Meta Search Engine(experiment version)',
	'author': 'Saeed, Kaushik',
	'version': '0.5',
	'description': 'Iris is a built-in meta search engine.',
	'comments': ('It should be note that this is a beta version and has many bugs!',),
	'contributors': 'Aman, Dimitris, Divya, Vikas, Kunal',
	'sources': ('google', 'bing', 'duckduckgo', 'etools', 'startpage', 'searx', 'yahoo'),
	'options': (
		('query', None, True, 'Query string', '-q', 'store', str),
	),
	'examples': ('iris -q <QUERY>',)
}

RESULTS = {}
MAPPED = {'google': 100, 'bing': 100, 'duckduckgo': 30,
	 'yahoo': 100, 'carrot2': 100, 'searx': 100, 'startpage': 30}

def thread(self, function, query, limit, workers):
	threadpool = concurrent.futures.ThreadPoolExecutor(max_workers=len(workers))
	futures = (threadpool.submit(function, self, x, query, limit) for x in workers)
	for _ in concurrent.futures.as_completed(futures):
		pass

def search(self, name, q, limit):
	global RESULTS
	count = MAPPED[name]
	try:
		engine = getattr(self, name)
		q = q
		varnames = engine.__init__.__code__.co_varnames
		if 'limit' in varnames and 'count' in varnames:
			attr = engine(q, limit, count)
		elif 'limit' in varnames:
			attr = engine(q, limit)
		else:
			attr = engine(q)
		attr.run_crawl()
		RESULTS[name] = attr.results
	except Exception as e:
		print(e)

def module_api(self):
	global RESULTS
	query = self.options['query']
	engines = MAPPED.keys()
	thread(self, search, query, 3, engines)
	simple_merge = self.meta_search_util.simple_merge([RESULTS[x] for x in engines if x in RESULTS])
	# TODO: Removing duplicates need to be done during merging
	final_results = self.meta_search_util.remove_dups(self, simple_merge)
	output = {'results': final_results}
	self.save_gather(output, 'iris/iris', query, output=self.options['output'])
	return output

def module_run(self):
	print()
	self.search_engine_results(module_api(self))



"""
OWASP Maryam!
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

meta = {
	'name': 'Find Social Networks',
	'author': 'Saeed',
	'version': '1.5',
	'description': 'Search to find Usernames in social networks.',
	'sources': ('bing', 'google', 'startpage', 'urlscan'),
	'options': (
		('query', None, True, 'Company Name or Query', '-q', 'store', str),
		('engines', 'google,bing', False, 'Search engine names. e.g `bing,google,..`', '-e', 'store', str),
		('url', None, False, 'The second source to crawl for', '-u', 'store', str),
		('depth', 1, False, 'Scraper depth level(default=1)(Only if the url is set)', '-d', 'store', int),
		('cthread', 5, False, 'The number of links that open per round(default=5)(Only if the url is set)', '-T', 'store', int),
		('limit', 5, False, 'Search limit(number of pages, default=5)', '-l', 'store', int),
		('count', 100, False, 'number of results per page(min=10, max=100, default=100)', '-c', 'store', int),
		('thread', 2, False, 'The number of engine that run per round(default=2)', '-t', 'store', int),
	),
	'examples': ('social_nets -q microsoft -e google,bing,yahoo -c 50 -t 3 --output',
		'social_nets -q microsoft -e google',
		'social_nets -q microsoft -u microsoft.com -e google,bing,yahoo, -c 50 -t 5 -T 10 -d 2')
}

PAGES = ''
def search(self, name, q, q_formats, limit, count):
	global PAGES
	engine = getattr(self, name)
	eng = name
	varnames = engine.__init__.__code__.co_varnames
	if 'limit' in varnames and 'count' in varnames:
		attr = engine(q, limit, count)
	elif 'limit' in varnames:
		reg_dom = self.reglib().domain_m
		if eng == 'urlscan':
			if not re.search(reg_dom, q):
				self.verbose('Invalid domain name. Cannot run urlscan')
				return
			else:
				attr = engine(q, limit)
	else:
		attr = engine(q)

	attr.run_crawl()
	PAGES += attr.pages

def module_api(self):
	global PAGES
	query = '@' + self.options['query']
	limit = self.options['limit']
	count = self.options['count']
	url = self.options['url']
	depth = self.options['depth']
	crawl_thread = self.options['cthread']
	engines = self.options['engines'].lower().split(',')
	page = ''
	try:
		if '.' in self.options['query']:
			page = self.request(self.options['query']).text
	except:
		page = ''
	PAGES += page
	self.thread(search, self.options['thread'], engines, query, {}, limit, count, meta['sources'])
	if url:
		scrap = self.web_scrap(url, False, depth, crawl_thread)
		scrap.run_crawl()
		PAGES += scrap.pages
	usernames = self.page_parse(PAGES).get_networks
	self.save_gather(usernames, 'osint/social_nets', query, output=self.options['output'])
	return usernames

def module_run(self):
	usernames = module_api(self)
	for net in usernames:
		lst = list(set(usernames[net]))
		if lst != []:
			self.alert(net)
			for atom in lst:
				self.output(f"\t{atom}", 'G')
